import 'package:flutter/material.dart';
import 'package:vaycore/utils.dart';
import 'package:vaycore/widget/net_image.dart';

//
// @author vay
// @email hxhvay@126.com
// @date 2020-05-20
//

/// 头像组件
class Avatar extends StatelessWidget {
  final NetImage _image;
  final Color borderColor;
  final double borderWidth;
  final EdgeInsetsGeometry margin;
  final AvatarShape shape;
  final double radius;

  Avatar(
    String src, {
    Key key,
    double size = 40,
    BoxFit fit = BoxFit.cover,
    this.borderColor = Colors.transparent,
    this.borderWidth = 0,
    this.margin = EdgeInsets.zero,
    this.shape = AvatarShape.circle,
    this.radius = 5,
  })  : _image = NetImage(src, width: size, height: size, fit: fit),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        shape: this._hasCircle ? BoxShape.circle : BoxShape.rectangle,
        borderRadius:
            this._hasCircle ? null : BorderRadius.circular(this.radius),
        color: this.borderColor,
      ),
      padding: EdgeInsets.all(this.borderWidth),
      margin: this.margin,
      child: _buildChild(),
    );
  }

  bool get _hasCircle => this.shape == AvatarShape.circle;

  Widget _buildChild() {
    return this._hasCircle ? _buildOvalAvatar() : _buildRoundAvatar();
  }

  Widget _buildOvalAvatar() {
    return ClipOval(
      child: WidgetUtils.notNull(this._image),
    );
  }

  Widget _buildRoundAvatar() {
    return ClipRRect(
      borderRadius: BorderRadius.circular(this.radius),
      child: WidgetUtils.notNull(this._image),
    );
  }
}

/// 头像形状
enum AvatarShape {
  /// 圆形
  circle,

  /// 圆角
  round,
}
