import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';

//
// @author vay
// @email hxhvay@126.com
// @date 2020-06-15
//

/// 网络图片
class NetImage extends StatelessWidget {
  final ImageProvider image;
  final ImageFrameBuilder frameBuilder;
  final ImageLoadingBuilder loadingBuilder;
  final ImageErrorWidgetBuilder errorBuilder;
  final double width;
  final double height;
  final Color color;
  final FilterQuality filterQuality;
  final BlendMode colorBlendMode;
  final BoxFit fit;
  final AlignmentGeometry alignment;
  final ImageRepeat repeat;
  final Rect centerSlice;
  final bool matchTextDirection;
  final bool gaplessPlayback;
  final String semanticLabel;
  final bool excludeFromSemantics;
  final bool hasCache;

  NetImage(
    String url, {
    Key key,
    double scale = 1.0,
    this.frameBuilder,
    this.loadingBuilder,
    this.errorBuilder,
    this.semanticLabel,
    this.excludeFromSemantics = false,
    this.width,
    this.height,
    this.color,
    this.colorBlendMode,
    this.fit = BoxFit.cover,
    this.alignment = Alignment.center,
    this.repeat = ImageRepeat.noRepeat,
    this.centerSlice,
    this.matchTextDirection = false,
    this.gaplessPlayback = false,
    this.filterQuality = FilterQuality.low,
    Map<String, String> headers,
    int cacheWidth,
    int cacheHeight,
    this.hasCache = true,
  })  : image = _getImageProvider(
            cacheWidth, cacheHeight, hasCache, url, scale, headers),
        assert(alignment != null),
        assert(repeat != null),
        assert(matchTextDirection != null),
        assert(cacheWidth == null || cacheWidth > 0),
        assert(cacheHeight == null || cacheHeight > 0),
        super(key: key);

  static ImageProvider _getImageProvider(
    int cacheWidth,
    int cacheHeight,
    bool hasCache,
    String url,
    double scale,
    Map<String, String> headers,
  ) {
    return ResizeImage.resizeIfNeeded(
      cacheWidth,
      cacheHeight,
      hasCache
          ? CachedNetworkImageProvider(url, scale: scale, headers: headers)
          : NetworkImage(url, scale: scale, headers: headers),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Image(
        image: this.image,
        key: this.key,
        frameBuilder: this.frameBuilder,
        loadingBuilder: this.loadingBuilder,
        errorBuilder: this.errorBuilder,
        semanticLabel: this.semanticLabel,
        excludeFromSemantics: this.excludeFromSemantics,
        width: this.width,
        height: this.height,
        color: this.color,
        colorBlendMode: this.colorBlendMode,
        fit: this.fit,
        alignment: this.alignment,
        repeat: this.repeat,
        centerSlice: this.centerSlice,
        matchTextDirection: this.matchTextDirection,
        gaplessPlayback: this.gaplessPlayback,
        filterQuality: this.filterQuality);
  }
}
